package Finance::Dogecoin;
$Finance::Dogecoin::VERSION = '1.20140201.1608';
use strict;
use warnings;

use Finance::Dogecoin::API;

'to the moon';
__END__
=pod

=head1 NAME

Finance::Dogecoin - use the dogeapi.com API from Perl

=head1 SYNOPSIS

See L<Finance::Dogecoin::API>.

=head1 COPYRIGHT & LICENSE

Copyright 2014 chromatic, some rights reserved.

This program is free software. You can redistribute it and/or modify it under
the same terms as Perl 5.18.

=cut
